;------------------------------------------------------------------------------
; Externa PS/2 klavesnica pre PMD 85 - RomBorSoft (c) 2009-2011

; Mikroprocesor: ATMEL ATtiny2313, 20 MHz
; PS/2 klavesnica musi poznat Scan kod KEY-SET-3 (mod bez rozsirenych kodov)


; Inspiracia / zdroje:
; http://www.computer-engineering.org/ps2protocol/
; http://www.beyondlogic.org/keyboard/keybrd.htm
; http://www.elektronika.kvalitne.cz/ATMEL/necoteorie/tutorial/PS2/PS2_mouse.html
; http://www.simandl.cz/stranky/elektro/keyboard/keyboard.htm
; David Matousek - Prace s mikrokontrolery ATMEL 2. dil, kap. 9.2
; Jiri Hrbacek - Komunikace mikrokontroleru s okolim 1. dil, kap.  2.2.6
; zdrojovy kod v jazyku C pre obsluhu PS/2 klavesnice pre nejaky ATMEL AVR,
;   z nejakeho klonu ZX Spectra - link som stratil


;------------------------------------------------------------------------------
	.nolist
	.include "tn2313def.inc"
	.list

;------------------------------------------------------------------------------
	.equ	XTAL	= 20000000	; frekvencia krystalu

	.equ	USELED	= 1

	; definicia roznych konstant
	.equ	KEYSL	= $5F	; zamena Z<->Y (klavesa Scroll Lock)
	.equ	KEYY	= $35	; klavesa Y
	.equ	KEYZ	= $1A	; klaveza Z
	.equ	KEYCL	= $14	; klavesa Caps Lock
	.equ	KEYCTR	= $11	; klavesa Lavy Ctrl
	.equ	KEYALT	= $19	; klavesa Lavy Alt
	.equ	KEYDEL	= $64	; klavesa Del
	.equ	KEYESC	= $08	; klavesa Esc

	.equ	RELEASE	= $0F0	; uvolnenie klavesy

	.equ	LEDSTS	= $0ED	; LED diody
	.equ	KBDRES	= $0FF	; RESET klavesnice
	.equ	RESEND	= $0FE	; vyziadanie naposledy posielaneho bytu

	.equ	KBDID	= $0F2	; ziskanie ID klavesnice
	.equ	KBDID1	= $0AB	; 1. ID kod klavesnice
	.equ	KBDID2	= $83	; 2. ID kod klavesnice

	.equ	SCANCD	= $0F0	; nastavenie Scan kodu
	.equ	KBDRPT	= $0F3	; nastavenie opakovania klavesnice
	.equ	KBDBRK	= $0F8	; nastavenie posielania Break kodov

	.equ	KBDACK	= $0FA	; ACK z klavesnice
	.equ	BATOK	= $0AA	; vysledok BAT (self-test) bol OK
	.equ	BATERR	= $0FC	; vysledok BAT (self-test) bol Error

	; definicia portov
	.equ	ROWS	= PORTB	; riadky PMD 85 klavesnice
	.equ	DROWS	= DDRB

	.equ	COL	= PIND	; stlpec PMD 85 klavesnice
	.equ	DCOL	= DDRD
	.equ	WRCOL	= 3	; zapis stlpca klavesnice - je na /INT1

	.equ	KBD	= PIND	; PS/2 klavesnica
	.equ	DKBD	= DDRD
	.equ	PKBD	= PORTD
	.equ	KBDCLK	= 2	; CLK klavesnice - je na /INT0
	.equ	KBDDAT	= 6	; DATA klavesnice

	; definicia premennych umiestnenych do registrov
	.def	KBDCOD	= r25	; kod stlacenej klavesy
	.def	KEYEXT	= r24	; stav klaves RESET, STOP, SHIFT

	.equ	BRESET	= 7
	.equ	BSTOP	= 6
	.equ	BSHIFT	= 5

	.def	FLAGS	= r23	; rozne priznaky

	.equ	ISALT	= 0	; stlacena klavesa ALT
	.equ	ISCTRL	= 1	; stlacena klavesa CTRL
	.equ	ISDEL	= 2	; stlacena klavesa DEL
	.equ	ISREL	= 3	; priznak uvolnenia klavesy
	.equ	KBDPAR	= 4	; priznak urcujuci paritu prijimaneho bytu
	.equ	ISACK	= 5	; priznak prijatia ACK z klavesnice
	.equ	ISBAT	= 6	; priznak prijatia BAT-AA kodu z klavesnice
	.equ	EXCHZY	= 7	; priznak vymeny klaves Z a Y

.IF	USELED
	.def	XLED	= r22	; maska rozvietenych LED na klavesnici

	.equ	SCRLCK	= 0	; LED ScrollLock signalizuje vymenu Z a Y
	.equ	NUMLCK	= 1
	.equ	CPSLCK	= 2
.ENDIF

	.def	YCOL	= r21	; posledne adresovany stlpec
	.def	WCOLT	= r20	; pracovny register pri zapise stlpca klavesnice

	.def	KEYTCK	= r19	; stavove pocitadlo prijmu a vysielania bytu
	.def	KEYWRK	= r10	; kod klavesy, ktory sa prijima / vysiela
	.def	KEYSND	= r9	; byte, ktory sa ma odoslat do klavesnice
	.def	KBUFPOS	= r8	; index do klavesnicoveho buffra
	.equ	KBUFSIZ	= 32	; velkost klavesnicoveho buffra


;------------------------------------------------------------------------------
	.dseg

; Tabulka matice klavesnice potrebuje 16 bytov, rozlozena je vsak v 52 bytoch
; RAM. Je to preto, ze adresovanie stlpca klavesnice na porte PD je na bitoch
; 0, 1, 4 a 5 a kvoli co najrychlejsiemu ziskaniu hodnoty stlpca, sa hodnota
; z portu neupravuje na rozsah 0 az 15, ale iba maskuje na rozsah 0 az 51.
; Z tohto rozsahu su vyuzite iba 4 skupiny po 4 bytoch a diery o velkosti 12
; bytov medzi skupinami mozu byt vyuzite na dalsie premenne.
KEYCOL:	.byte	4		; stlpce klavesnice 0 az 3
	.byte	12		; diera
	.byte	4		; stlpce klavesnice 4 az 7
	.byte	12		; diera
	.byte	4		; stlpce klavesnice 8 az 11
	.byte	12		; diera
	.byte	4		; stlpce klavesnice 12 az 15

; buffer prijatych kodov z klavesnice.
KEYBUF:	.byte	KBUFSIZ


;------------------------------------------------------------------------------
	.cseg

	.org	0
	rjmp	RESET

	.org	INT0addr	; prerusenie od signalu CLK z PS/2 klavesnice
	rjmp	INT_KBD_CLK

	.org	INT1addr	; prerusenie pri zapise stlpca na port PA 8255
	rjmp	INT_WRITE_COL


;------------------------------------------------------------------------------
	.db	"Externa PS/2 klavesnica pre PMD 85 v2.0",0
	.db	"RomBorSoft (c) 2009-2011",0,0


;------------------------------------------------------------------------------
RESET:	ldi	r16,low(RAMEND)		; zasobnik na koniec pamati
	out	SPL,r16

	clr	r16			; port D - vstupy s Pull-up
	out	DDRD,r16
	ser	r16
	out	PORTD,r16

	out	ROWS,r16		; port B ako vystupy s urovnou H
	out	DROWS,r16

	cbi	PKBD,KBDCLK		; zablokuj CLK klavesnice 
	sbi	DKBD,KBDCLK
	cbi	PKBD,KBDDAT		; DATA klavesnice do Z stavu

	ldi	r16,(1<<ISC01)|(1<<ISC11) ; externe prerusenia budu citlive
	out	MCUCR,r16		; na zostupnu hranu

	ldi	r16,(1<<INT0)|(1<<INT1)	; povolenie externych preruseni
	out	GIMSK,r16

.IF	USELED
	ldi	XLED,(1<<NUMLCK)	; NUM LOCK
.ENDIF

	clr	XH			; vynuluj vysie byty ukazatelov X a Y
	clr	YH

	ldi	r16,150			; pockaj 1,5s
	rcall	DELAY_VAR

	clr	r16			; zrus pripadne nastavene flagy
	out	EIFR,r16		; prerusenia

	clr	FLAGS

	sei				; povolenie prerusenia

	cbi	DKBD,KBDCLK		; uvolni CLK klavesnice 

KBD_RESET:
	rcall	RESET_KEYBOARD		; zresetuj PS/2 klavesnicu


;----------------
; Slucka testu klavesnice.
; ALT - nuluje maticu
; CTRL+ALT+DEL - RESET pocitaca (nuluje maticu)
; CTRL+SHIFT+ESC - RESET klavesnice (nuluje maticu)
; ScrollLock - zamiena Z a Y
KBD_LOOP:
	tst	KBUFPOS			; je v buffri pripraveny byte ?
	breq	KBD_LOOP		; nie, cakaj

	cli				; zakaz na chvilu prerusenie
	ldi	XL,KEYBUF		; do X bazova adresa buffra
	ld	KBDCOD,X		; prevezmi aktualny kod
	clr	ZH			; do Z bazova adresa buffra + 1
	ldi	ZL,KEYBUF+1
	mov	r16,KBUFPOS		; preved posun poloziek v buffri
KBD_BUFF_1:				; na zaciatok buffra
	dec	r16
	breq	KBD_BUFF_2
	ld	r15,Z+
	st	X+,r15
	rjmp	KBD_BUFF_1
KBD_BUFF_2:
	dec	KBUFPOS			; zniz pocet poloziek v buffri
	sei				; povol prerusenie

	set
;----
KBD_RELEASE:
	cpi	KBDCOD,RELEASE
	brne	KBD_ESCAPE
	bld	FLAGS,ISREL		; je to kod uvolnenia klavesy
	rjmp	KBD_LOOP
;----
KBD_ESCAPE:
	cpi	KBDCOD,KEYESC		; Esc
	brne	KBD_ALT
	sbrc	FLAGS,ISREL		; iba pri stlaceni
	rjmp	KBD_LOOP

	mov	r16,KEYEXT
	andi	r16,(1<<BSTOP)|(1<<BSHIFT) ; CTRL+SHIFT+Esc - RESET KBD
	cpi	r16,(1<<BSTOP)|(1<<BSHIFT)
	breq	KBD_RESET
	rjmp	KBD_LOOP
;----
KBD_ALT:
	cpi	KBDCOD,KEYALT		; Alt - vynulovanie KBD matice
	brne	KBD_DEL
	sbrc	FLAGS,ISREL
	rjmp	KBD_ALT_REL
;	rcall	CLEAR_MATRIX		; vynuluj maticu klaves
	set
	bld	FLAGS,ISALT
	clt
	bld	FLAGS,ISDEL
	rjmp	KBD_EXT_OFF

KBD_ALT_REL:
;	rcall	CLEAR_MATRIX		; vynuluj maticu klaves
	clt
	bld	FLAGS,ISALT
	bld	KEYEXT,BRESET
	rjmp	KBD_EXT_OFF
;----
KBD_DEL:
	cpi	KBDCOD,KEYDEL		; Del
	brne	KBD_CAPS_LOCK
	sbrc	FLAGS,ISREL
	rjmp	KBD_DEL_REL
	bld	FLAGS,ISDEL
	sbrs	KEYEXT,BSTOP
	rjmp	KBD_CODE_FIND
	sbrs	FLAGS,ISALT
	rjmp	KBD_CODE_FIND
	bld	KEYEXT,BRESET		; RESET
	cbi	ROWS,BRESET
	rjmp	KBD_EXT_OFF

KBD_DEL_REL:
	clt
	bld	FLAGS,ISDEL
	bld	KEYEXT,BRESET
	sbi	ROWS,BRESET
	andi	XLED,~(1<<CPSLCK)	; RESET pocitaca rusi CapsLock
	rcall	LED_ON
	rjmp	KBD_CODE_FIND
;----
KBD_CAPS_LOCK:
.IF	USELED
	cpi	KBDCOD,KEYCL		; Caps Lock
	brne	KBD_SCROLL_LOCK
	sbrc	FLAGS,ISREL		; iba pri stlaceni
	rjmp	KBD_CODE_FIND
	ldi	r16,(1<<CPSLCK)
	eor	XLED,r16
	rcall	LED_ON
	rjmp	KBD_CODE_FIND
.ENDIF
;----
KBD_SCROLL_LOCK:
	cpi	KBDCOD,KEYSL
	brne	KBD_EXCHANGE		; ScrollLock meni Z a Y
	sbrc	FLAGS,ISREL		; iba pri stlaceni
	rjmp	KBD_EXT_OFF
	ldi	r16,(1<<EXCHZY)
	eor	FLAGS,r16		; zamena Z a Y
.IF	USELED
	ldi	r16,(1<<SCRLCK)
	eor	XLED,r16
	rcall	LED_ON
.ENDIF
	rjmp	KBD_EXT_OFF
;----
KBD_EXCHANGE:
	sbrs	FLAGS,EXCHZY
	rjmp	KBD_CODE_FIND
	cpi	KBDCOD,KEYY
	brne	KBD_EXCHANGE_Z
	ldi	KBDCOD,KEYZ
	rjmp	KBD_CODE_FIND

KBD_EXCHANGE_Z:
	cpi	KBDCOD,KEYZ
	brne	KBD_CODE_FIND
	ldi	KBDCOD,KEYY

;----
; vyhladanie kodu v tabulkach
KBD_CODE_FIND:
	cpi	KBDCOD,$90		; max. kod z klavesnice je 08Fh
	brcc	KBD_EXT_OFF
	ldi	ZH,high(ATAB1)		; adresa tabulky kodov do Z
	ldi	ZL,low(ATAB1)
KBD_CODE_EXT:
	clr	r16
	add	ZL,KBDCOD
	adc	ZH,r16
	add	ZL,KBDCOD
	adc	ZH,r16			; Z = Z + 2 * KBDCOD
	lpm
	mov	r6,r0			; cislo stlpca do r6
	adiw	ZL,1
	lpm
	mov	r16,r0			; maska v r0 a r16

KBD_CODE_STOP:
	lsl	r0
	lsl	r0
	brcc	KBD_CODE_SHIFT
	set
	sbrc	FLAGS,ISREL
	clt
	bld	KEYEXT,BSTOP		; STOP
	sbi	ROWS,BSTOP
	sbrc	KEYEXT,BSTOP
	cbi	ROWS,BSTOP

KBD_CODE_SHIFT:
	lsl	r0
	brcc	KBD_CODE_ALL
	set
	sbrc	FLAGS,ISREL
	clt
	bld	KEYEXT,BSHIFT		; SHIFT
	sbi	ROWS,BSHIFT
	sbrc	KEYEXT,BSHIFT
	cbi	ROWS,BSHIFT

KBD_CODE_ALL:
	andi	r16,$1F
	breq	KBD_EXT_OFF
	mov	r7,r16

	ldi	XL,KEYCOL		; bazova adresa tabulky
	add	XL,r6			; pripocitaj cislo stlpca
	ld	r16,X			; precitaj stav stlpca
	sbrc	FLAGS,ISREL
	rjmp	KBD_CODE_REL

	; stlacenie klavesy
	or	r16,r7			; pridaj masku klavesy
	rjmp	KBD_CODE_STORE

KBD_CODE_REL:				; uvolnenie klavesy
	com	r7			; invertuj masku klavesy
	and	r16,r7			; odmaskuj klavesu
KBD_CODE_STORE:
	st	X,r16			; uloz novy stav

;	sbrs	FLAGS,ISREL
;	rjmp	KBD_EXT_OFF

	cli
	cpse	r6,YCOL
	rjmp	KBD_EXT_OFF

	or	r16,KEYEXT		; pridaj k stavu stlpca stav SHIFT/STOP
	com	r16			; komplement hodnoty
	out	ROWS,r16		; posli na vystup

KBD_EXT_OFF:
	sei

	clt
	bld	FLAGS,ISREL		; zrus flagy

	rjmp	KBD_LOOP


;------------------------------------------------------------------------------
; Vynuluje tabulku matice klavesnice i niektore premenne a priznaky.
; M: r14,r17,X
CLEAR_MATRIX:
	ser	r17			; vystupy do H
	out	ROWS,r17
	ser	YCOL			; zneplatni posledny pouzity stlpec

	clr	r14			; nulovaci register
	ldi	r17,52			
	ldi	XL,KEYCOL		; bazova adresa tabulky matice kl.
CLEAR_MATRIX_L:
	st	X+,r14			; vynuluj tabulku matice klavesnice
	dec	r17
	brne	CLEAR_MATRIX_L

	andi	FLAGS,(1<<EXCHZY)	; incializuj flagy a premenne
	clr	KEYEXT
	clr	KBDCOD

	clr	KEYTCK
	clr	KEYWRK
	clr	KEYSND
	clr	KBUFPOS
	ret


;------------------------------------------------------------------------------
; Rozsvieti alebo zhasne klavesnicove LED podla hodnoty premennej XLED.
; M: r16
.IF	USELED
LED_ON:	ldi	r16,LEDSTS		; povel pre zosvietenie/zhasnutie LED
	rcall	KBD_SEND_BYTE
	rcall	WAIT_ACK		; pockaj na ACK
	mov	r16,XLED		; posli pozadovany stav LED
	rcall	KBD_SEND_BYTE
	rjmp	WAIT_ACK		; pockaj na ACK
.ENDIF


;------------------------------------------------------------------------------
; Prevedie reset klavesnice, nastavi ScanCode set 3, posielanie Break kodu
; a pomale opakovanie klavesnice.
RESET_KEYBOARD:
	rcall	CLEAR_MATRIX		; vynuluj klavesnicovu maticu
					; i niektore premenne a flagy

	ldi	r16,10			; pauza 100ms
	rcall	DELAY_VAR

	ldi	r16,KBDRES		; povel RESET do klavesnice
	rcall	KBD_SEND_BYTE

	ldi	r17,200			; max. 2 s cakaj na ukocenie BAT
RESET_KEYBOARD_1:
	ldi	r16,1			; pauza 10ms
	rcall	DELAY_VAR

	sbrs	FLAGS,ISACK		; poslala klavesnica ACK ?
	rjmp	RESET_KEYBOARD_2	; ak nie, skoc dalej
	sbrc	FLAGS,ISBAT		; poslala klavesnica BAT OK
	rjmp	RESET_KEYBOARD_3	; ak ano, skoc dalej

RESET_KEYBOARD_2:
	dec	r17			; zniz pocitadlo
	brne	RESET_KEYBOARD_1	; cakaj dalej

	rjmp	RESET_KEYBOARD		; pokial sa reset nepodaril, skus znovu

RESET_KEYBOARD_3:
	ldi	r16,SCANCD		; povel - nastavenie scan-code setu
	rcall	KBD_SEND_BYTE
	rcall	WAIT_ACK		; pockaj na ACK
	ldi	r16,3			; Scan code set 3
	rcall	KBD_SEND_BYTE
	rcall	WAIT_ACK		; pockaj na ACK

	ldi	r16,KBDBRK		; povel - zapni posielanie Break kodu
	rcall	KBD_SEND_BYTE
	rcall	WAIT_ACK		; pockaj na ACK

	ldi	r16,KBDRPT		; povel opakovanie klaves
	rcall	KBD_SEND_BYTE
	rcall	WAIT_ACK		; pockaj na ACK
	ldi	r16,0b01111111		; opakovanie po 1s, 2 zn./s
	rcall	KBD_SEND_BYTE
	rcall	WAIT_ACK		; pockaj na ACK

	ldi	r16,100			; pauza 1s
	rcall	DELAY_VAR

.IF	USELED
	rcall	LED_ON
.ENDIF
	ret


;------------------------------------------------------------------------------
; Pocka na byte ACK z klavesnice. Caka sa max. 1s
; I: -
; O: T=1, ak ACK bolo prijate, T=0, ak prijate do 1s nebolo
; M: r16, r17
WAIT_ACK:
	clt
	bld	FLAGS,ISACK		; vynuluj priznak ACK

	ldi	r17,100			; max 1 s cakaj na ACK
WAIT_ACK_1:
	ldi	r16,1			; pauza 10ms
	rcall	DELAY_VAR

	sbrc	FLAGS,ISACK		; poslala klavesnica ACK ?
	rjmp	WAIT_ACK_2		; ak ano, skoc dalej

	dec	r17			; zniz pocitadlo
	brne	WAIT_ACK_1		; cakaj dalej

WAIT_ACK_2:
	bst	FLAGS,ISACK		; do T uloz stav ACK
	ret


;------------------------------------------------------------------------------
; Rutina pripravi byte na odoslanie do klavesnice.
; I: r16=byte na odoslanie
; O: -
; M: SREG
KBD_SEND_BYTE:
	tst	KEYTCK			; cakaj kym sa ukonci prijimanie
	brne	KBD_SEND_BYTE		; alebo odosielanie

	sbi	DKBD,KBDCLK		; zablokuj CLK klavesnice

	rcall	DELAY_100		; pockaj 100us

	clt
	bld	FLAGS,ISACK		; nuluj flagy
	bld	FLAGS,ISBAT

	mov	KEYSND,r16		; priprav posielany byte
	ldi	KEYTCK,12		; stavove pocitadlo nastav pre odosielanie

	cbi	PKBD,KBDDAT		; ziadost o vysielanie - START bit
	sbi	DKBD,KBDDAT

	cbi	DKBD,KBDCLK		; uvolni CLK klavesnice 
	ret


;------------------------------------------------------------------------------
; Prerusenie vyvolane zostupnou hranou hodin PS/2 klavesnice.
; Vykonava postupne prijimanie a odosielanie bytu z/do PS/2 klavesnice.
; M: Y, r3
INT_KBD_CLK:
	in	r3,SREG			; odpamataj stavovy register
	push	r16

	rcall	DELAY_10		; pockaj 10us

	sbic	KBD,KBDCLK		; CLK musi byt stale v L
	rjmp	INT_KBD_END		; ak nie je, asi to bol zakmit, vrat sa

	cpi	KEYTCK,12		; odosiela sa byte ?
	brcc	INT_KBD_GT10		; ano, skoc dalej

	clt				; stav KBDDAT do priznaku T
	sbic	KBD,KBDDAT
	set

	; prijem bytu
	tst	KEYTCK			; caka sa start bit?
	brne	INT_KBD_GT0		; nie, skoc dalej
	brts	INT_KBD_END2		; ak to nie je start bit (0), vrat sa
	clr	KEYWRK			; vynuluj pracovny byte
	inc	KEYTCK			; budeme cakat na prvy datovy bit
	bld	FLAGS,KBDPAR		; vynuluj paritny bit
	rjmp	INT_KBD_END		; a vrat sa

INT_KBD_GT0:
	cpi	KEYTCK,9		; cakaju sa datove bity ?
	brcc	INT_KBD_GT8		; nie, skoc dalej
	lsr	KEYWRK			; posun pracovny byte o bit doprava
	bld	KEYWRK,7		; pridaj datovy bit
	inc	KEYTCK			; budeme cakat na dalsi datovy bit
	brtc	INT_KBD_END2		; ak je to nulovy bit, hned sa vrat
	ldi	r16,1<<KBDPAR		; inak, complementuj paritny bit
	eor	FLAGS,r16
	rjmp	INT_KBD_END		; a vrat sa


INT_KBD_GT8:
	brne	INT_KBD_GT9		; ak to nie je paritny bit, skoc dalej
	inc	KEYTCK			; budeme cakat na stop bit
	brtc	INT_KBD_END2		; ak je to nulovy bit, hned sa vrat
	ldi	r16,1<<KBDPAR		; inak, complementuj paritny bit
	eor	FLAGS,r16
	rjmp	INT_KBD_END		; a vrat sa

INT_KBD_GT9:
	cpi	KEYTCK,10		; caka sa stop bit ?
	brne	INT_KBD_GT10		; nie, skoc dalej
	clr	KEYTCK			; vynuluj stavove pocitadlo
	sbrc	FLAGS,KBDPAR		; ak je parita v poriadku,
	rjmp	INT_KBD_PAR_OK		; skoc dalej
	ldi	r16,RESEND		; chyba parity
INT_KBD_PAR_OK_2:
	rcall	KBD_SEND_BYTE		; vyziadame si poslat kod este raz
	rjmp	INT_KBD_END		; vrat sa z prerusenia

INT_KBD_PAR_OK:
	; prijaty kod z klavesnice sa porovna na niektore kody
	; 0      - nic
	; RESEND - klavesnica ziada naposledy odoslany kod
	; KBDACK - klavesnica poslala potvrdzovaci kod ACK
	; BATOK  - klavesnica poslala BAT-OK kod po self-teste
	; ostatne kody sa vkladaju do klavesnicoveho buffra
	tst	KEYWRK			; je to 0 ?
	breq	INT_KBD_END		; ano, ignoruj kod a vrat sa z prerusenia
	set				; T=1
	ldi	r16,RESEND		; kod RESEND
	cp	KEYWRK,r16		; pozaduje klavesnica znovu o posledny byte?
	mov	r16,KEYSND		; naposledy posielany byte
	breq	INT_KBD_PAR_OK_2	; ano, skoc ho poslat
	ldi	r16,KBDACK		; kod ACK
	cp	KEYWRK,r16		; je to potvrdzovaci kod?
	brne	INT_KBD_PAR_OK_3	; nie, skoc dalej
	bld	FLAGS,ISACK		; nastav flag
	rjmp	INT_KBD_END		; vrat sa z prerusenia

INT_KBD_PAR_OK_3:
	ldi	r16,BATOK		; kod BAT_OK
	cp	KEYWRK,r16		; je to BAT-OK kod?
	brne	INT_KBD_PAR_OK_4	; nie, skoc dalej
	bld	FLAGS,ISBAT		; nastav flag
	rjmp	INT_KBD_END		; vrat sa z prerusenia

INT_KBD_PAR_OK_4:
	; pridanie kodu do klavesnicoveho buffra
	ldi	r16,KBUFSIZ		; zisti, ci je miesto v buffri
	cp	KBUFPOS,r16
	breq	INT_KBD_END		; nie je, vrat sa
	ldi	YL,KEYBUF		; bazova adresa buffra
	add	YL,KBUFPOS		; pripocitaj poziciu v buffri
	st	Y,KEYWRK		; uloz prijaty kod do buffra
	inc	KBUFPOS			; zvys index do buffra
INT_KBD_END2:
	rjmp	INT_KBD_END		; vrat sa z prerusenia

	; odosielanie bytu
INT_KBD_GT10:
	cpi	KEYTCK,20		; odosiela sa byte ?
	brcc	INT_KBD_GT19		; nie, skoc dalej
	cpi	KEYTCK,12		; je to nulty bit?
	brne	INT_KBD_GT12		; nie, skoc dalej
	mov	KEYWRK,KEYSND		; priprav byte na odoslanie
	ori	FLAGS,(1<<KBDPAR)	; inicializuj paritu
INT_KBD_GT12:
	lsr	KEYWRK			; posun pracovny byte o bit doprava
	brcc	INT_KBD_GT12_0
	sbi	PKBD,KBDDAT		; log. 1 na KBDDAT
	ldi	r16,1<<KBDPAR		; complementuj paritny bit
	eor	FLAGS,r16
	rjmp	INT_KBD_GT12_1
INT_KBD_GT12_0:
	cbi	PKBD,KBDDAT		; log. 0 na KBDDAT
INT_KBD_GT12_1:
	inc	KEYTCK
	rjmp	INT_KBD_END		; vrat sa z prerusenia

INT_KBD_GT19:
	brne	INT_KBD_GT20		; ak to nie je paritny bit, skoc dalej
	sbrs	FLAGS,KBDPAR		; podla parity
	cbi	PKBD,KBDDAT		; log. 0
	sbrc	FLAGS,KBDPAR
	sbi	PKBD,KBDDAT		; log. 1
	inc	KEYTCK
	rjmp	INT_KBD_END		; vrat sa z prerusenia

INT_KBD_GT20:
	cpi	KEYTCK,21		; ukoncenie odosielania ?
	brne	INT_KBD_GT21
	cbi	DKBD,KBDDAT		; uvolni DATA klavesnice - STOP bit
	sbi	PKBD,KBDDAT
	inc	KEYTCK
	rjmp	INT_KBD_END		; vrat sa z prerusenia

INT_KBD_GT21:
	cpi	KEYTCK,22		; koniec odosielania ?
	brne	INT_KBD_END
	clr	KEYTCK

INT_KBD_END:
	pop	r16
	out	SREG,r3			; obnov stavovy register
	reti


;------------------------------------------------------------------------------
; Prerusenie vyvolane zapisom na port $F4 (OUT 0F4H) - volba stlpca matice.
; Prevezme cislo stlpca matice a z tabulky vyberie stav tohto stlpca, prida.
; stav klaves SHIFT a STOP a posle na vystupny port.
; Tabulka matice klavesnice ma zamerne namiesto 16-tich, 52 poloziek.
; Vyuzite su samozrejme iba tie, ktore zodpovedaju offsetom maske 0b00110011.
; M: Y, YCOL, WCOLT
INT_WRITE_COL:
	in	WCOLT,SREG		; stavovy register na zasobnik
	push	WCOLT
	in	WCOLT,COL		; prevezmi cislo stlpca xx32xx10
	andi	WCOLT,0b00110011	; odmaskuj platne bity
	mov	YCOL,WCOLT		; odpamataj cislo stlpca
	ldi	YL,KEYCOL		; bazova adresa tabulky
	add	YL,WCOLT		; pripocitaj cislo stlpca
	ld	WCOLT,Y			; precitaj stav stlpca
	or	WCOLT,KEYEXT		; pridaj k stavu stlpca stav SHIFT/STOP
	com	WCOLT			; complement hodnoty
	out	ROWS,WCOLT		; posli na vystup
	pop	WCOLT			; obnov stavovy register
	out	SREG,WCOLT
	reti				; navrat a povolenie prerusenia


;------------------------------------------------------------------------------
; zdrzanie 10us
; I: -
; O: -
; M: r1
DELAY_10:
	in	r1,SREG			; odpamataj stavovy register
	push	r16
	ldi	r16,XTAL/(3*100000)-3
DELAY_10_2:
	dec	r16
	brne	DELAY_10_2
	pop	r16
	out	SREG,r1			; obnov stavovy register
	ret


;------------------------------------------------------------------------------
; zdrzanie 100us
; I: -
; O: -
; M: r5
DELAY_100:
	in	r5,SREG			; odpamataj stavovy register
	push	r16
	ldi	r16,XTAL/(8*10000)-1
DELAY_100_2:
	push	r16
	pop	r16
	nop
	dec	r16
	brne	DELAY_100_2
	pop	r16
	out	SREG,r5			; obnov stavovy register
	ret


;------------------------------------------------------------------------------
; zdrzanie (r16 * 10ms)
; I: r16=zdrzanie v 1/100 sekundy
; O: r16=0
; M: r2, r4, r16
DELAY_VAR:
	in	r2,SREG			; odpamataj stavovy register
DELAY_VAR_3:
	ldi	r18,250
	mov	r4,r18
DELAY_VAR_2:
	ldi	r18,XTAL/(8*25000)
DELAY_VAR_1:
	push	r16
	pop	r16
	nop
	dec	r18
	brne	DELAY_VAR_1
	dec	r4
	brne	DELAY_VAR_2
	dec	r16
	brne	DELAY_VAR_3
	out	SREG,r2			; obnov stavovy register
	ret


;------------------------------------------------------------------------------
; tabulka zakladnych klaves
; * tvar tabulky => cislo stlpca v matici, maska riadku matice
; * poradie poloziek tabulky zodpoveda kodom klaves posielanych PS/2
;   klavesnicou pre KEY-SET 3 (mod bez rozsirenych kodov)
; * "prekladova" tabulka cisla stlpca do riadku matice je nasledujuca
; 0 -> 0, 1 -> 1, 2 -> 2, 3 -> 3, 4 -> 16, 5 -> 17, 6 -> 18, 7 -> 19
; 8 -> 32, 9 -> 33, 10 -> 34, 11 -> 35, 12 -> 48, 13 -> 49, 14 -> 50, 15 -> 51
TAB1:	.db	 0,   0	; 0
	.db	 0,   0
	.db	 0,   0
	.db	 0,   0
	.db	 0,   0
	.db	 0,   0
	.db	 0,   0
	.db	 0,   1		; K0		(F1)
	.db	 0,   0 ; 8	; Esc
	.db	 0,   0
	.db	 0,   0
	.db	 0,   0
	.db	 0,   0
	.db	49,   1		; C-D		(Tab)
	.db	48,   1		; WRK `
	.db	 1,   1		; K1		(F2)
	.db	 0,   0	; 10
	.db	 0,  64		; STOP		(lavy Ctrl)
	.db	 0,  32		; SHIFT		(lavy Shift)
	.db	 0,   0
	.db	48,  16		; CapsLock PMD 85-3
	.db	 0,   4		; Q
	.db	 0,   2		; 1
	.db	 2,   1		; K2		(F3)
	.db	 0,   0	; 18
	.db	 0,   0		; 		(lavy Alt)
	.db	17,   4		; Z
	.db	 1,   8		; S
	.db	 0,   8		; A
	.db	 1,   4		; W
	.db	 1,   2		; 2
	.db	 3,   1		; K3		(F4)
	.db	 0,   0	; 20
	.db	 3,  16		; C
	.db	 2,  16		; X
	.db	 2,   8		; D
	.db	 2,   4		; E
	.db	 3,   2		; 4
	.db	 2,   2		; 3
	.db	16,   1		; K4		(F5)
	.db	 0,   0	; 28
	.db	 0,  16		; medzera
	.db	16,  16		; V
	.db	 3,   8		; F
	.db	16,   4		; T
	.db	 3,   4		; R
	.db	16,   2		; 5
	.db	17,   1		; K5		(F6)
	.db	 0,   0	; 30
	.db	18,  16		; N
	.db	17,  16		; B
	.db	17,   8		; H
	.db	16,   8		; G
	.db	 1,  16		; Y
	.db	17,   2		; 6
	.db	18,   1		; K6		(F7)
	.db	 0,   0	; 38
	.db	 0,   0		; 		(pravy Alt)
	.db	19,  16		; M
	.db	18,   8		; J
	.db	18,   4		; U
	.db	18,   2		; 7
	.db	19,   2		; 8
	.db	19,   1		; K7		(F8)
	.db	 0,   0	; 40
	.db	32,  16		; ciarka ,
	.db	19,   8		; K
	.db	19,   4		; I
	.db	32,   4		; O
	.db	33,   2		; 0
	.db	32,   2		; 9
	.db	32,   1		; K8		(F9)
	.db	 0,   0	; 48
	.db	33,  16		; bodka .
	.db	34,  16		; lomitko /
	.db	32,   8		; L
	.db	33,   8		; bodkociarka ;
	.db	33,   4		; P
	.db	34,   2		; podtrzitko _ -
	.db	33,   1		; K9		(F10)
	.db	 0,   0	; 50
	.db	 0,   0
	.db	34,   8		; dvojbodka :
	.db	 0,   0
	.db	34,   4		; zavinac @
	.db	35,   8		; hranate [ ]
	.db	34,   1		; K10		(F11)
	.db	 0,   0		;		(PrintScreen)
	.db	 0,  64 ; 58	; STOP		(pravy Ctrl)
	.db	 0,  32		; SHIFT		(pravy Shift)
	.db	49,  16		; lavy EOL	(Enter)
	.db	35,   4		; opacne lomitko \
	.db	35,   2		; zlozene { }
	.db	 0,   0
	.db	35,   1		; K11		(F12)
	.db	 0,   0		; 		(Scroll Lock)
	.db	50,   8 ; 60	; -->I		(Down)
	.db	48,   4		; <--		(Left)
	.db	 0,   0		;		(Pause/Break)
	.db	48,   8		; I<--		(Up)
	.db	49,   2		; DEL		(Delete)
	.db	49,   8		; END		(End)
	.db	48,   4		; <--		(Back Space)
	.db	48,   2		; 70 - INS	(Insert)
	.db	 0,   0 ; 68
	.db	 0,   2		; 1		(Num 1)
	.db	50,   4		; -->		(Right)
	.db	 3,   2		; 4		(Num 4)
	.db	18,   2		; 7		(Num 7)
	.db	50,   2		; CLR		(Page Down)
	.db	49,   4		; HOME		(Home)
	.db	50,   1		; RCL		(Page Up)
	.db	33,   2	; 70	; 0		(Num 0)
	.db	33,  16		; .		(Num .)
	.db	 1,   2		; 2		(Num 2)
	.db	16,   2		; 5		(Num 5)
	.db	17,   2		; 6		(Num 6)
	.db	19,   2		; 8		(Num 8)
	.db	 0,   0		; 		(Num Lock)
	.db	34,  16		; /		(Num /) -- ??
	.db	 0,   0 ; 78
	.db	50,  16		; pravy EOL	(Num Enter)
	.db	 2,   2		; 3		(Num 3)
	.db	 0,   0
	.db	33,   8+32	; +		(Num +)
	.db	32,   2		; 9		(Num 9)
	.db	34,   8+32	; *		(Num *)
	.db	 0,   0		;		(Sleep)
	.db	 0,   0	; 80
	.db	 0,   0
	.db	 0,   0
	.db	 0,   0
	.db	33,   2+32	; -		(Num -) -- ??
	.db	 0,   0
	.db	 0,   0
	.db	 0,   0
	.db	 0,   0 ; 88
	.db	 0,   0
	.db	 0,   0
	.db	 0,   0		;		(lavy WIN)
	.db	 0,   0		;		(pravy WIN)
	.db	 0,   0		;		(APPS)
	.db	 0,   0
	.db	 0,   0	; 8F

	.equ	ATAB1	= (2*TAB1)

;------------------------------------------------------------------------------
	.EXIT
